import { r as registerInstance, c as createEvent, g as getContext, h, d as getElement } from './core-e0edbf68.js';
import './_commonjsHelpers-97e6d7b1.js';
import { l as lib_3, a as lib_2, b as lib_5, c as lib_4, d as lib_1 } from './index-74202bbc.js';
import { i as isArrayLike, a as isArray, b as isBuffer, c as isTypedArray, d as isArguments, g as getTag, e as isPrototype, f as baseKeys, h as baseIsEqual, j as baseGet, t as toKey, k as castPath, l as isObjectLike, m as baseGetTag, n as getPrototype, S as Symbol, o as arrayPush, p as defineProperty, q as identity, r as arrayMap, s as copyObject, u as getAllKeysIn, v as baseClone, w as isObject, x as isIndex, y as eq, z as isSymbol, P as PieLoader, V as VERSION, A as cloneDeep, B as parseNpm, C as pieContentFromConfig, D as complexRubricChecks, E as removeComplexRubricFromMarkup, F as COMPLEX_RUBRIC, G as addPackageToContent, H as addModelToContent, I as createTag, J as addComplexRubric, K as mathRenderingAccessible$1, L as addRubric, M as addMultiTraitRubric, N as DEFAULT_ENDPOINTS } from './version-2c94e221.js';

/** `Object#toString` result references. */
var mapTag = '[object Map]',
    setTag = '[object Set]';

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * Checks if `value` is an empty object, collection, map, or set.
 *
 * Objects are considered empty if they have no own enumerable string keyed
 * properties.
 *
 * Array-like values such as `arguments` objects, arrays, buffers, strings, or
 * jQuery-like collections are considered empty if they have a `length` of `0`.
 * Similarly, maps and sets are considered empty if they have a `size` of `0`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is empty, else `false`.
 * @example
 *
 * _.isEmpty(null);
 * // => true
 *
 * _.isEmpty(true);
 * // => true
 *
 * _.isEmpty(1);
 * // => true
 *
 * _.isEmpty([1, 2, 3]);
 * // => false
 *
 * _.isEmpty({ 'a': 1 });
 * // => false
 */
function isEmpty(value) {
  if (value == null) {
    return true;
  }
  if (isArrayLike(value) &&
      (isArray(value) || typeof value == 'string' || typeof value.splice == 'function' ||
        isBuffer(value) || isTypedArray(value) || isArguments(value))) {
    return !value.length;
  }
  var tag = getTag(value);
  if (tag == mapTag || tag == setTag) {
    return !value.size;
  }
  if (isPrototype(value)) {
    return !baseKeys(value).length;
  }
  for (var key in value) {
    if (hasOwnProperty.call(value, key)) {
      return false;
    }
  }
  return true;
}

var isEmpty_1 = isEmpty;

/**
 * Performs a deep comparison between two values to determine if they are
 * equivalent.
 *
 * **Note:** This method supports comparing arrays, array buffers, booleans,
 * date objects, error objects, maps, numbers, `Object` objects, regexes,
 * sets, strings, symbols, and typed arrays. `Object` objects are compared
 * by their own, not inherited, enumerable properties. Functions and DOM
 * nodes are compared by strict equality, i.e. `===`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
 * @example
 *
 * var object = { 'a': 1 };
 * var other = { 'a': 1 };
 *
 * _.isEqual(object, other);
 * // => true
 *
 * object === other;
 * // => false
 */
function isEqual(value, other) {
  return baseIsEqual(value, other);
}

var isEqual_1 = isEqual;

/**
 * Gets the last element of `array`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to query.
 * @returns {*} Returns the last element of `array`.
 * @example
 *
 * _.last([1, 2, 3]);
 * // => 3
 */
function last(array) {
  var length = array == null ? 0 : array.length;
  return length ? array[length - 1] : undefined;
}

var last_1 = last;

/**
 * The base implementation of `_.slice` without an iteratee call guard.
 *
 * @private
 * @param {Array} array The array to slice.
 * @param {number} [start=0] The start position.
 * @param {number} [end=array.length] The end position.
 * @returns {Array} Returns the slice of `array`.
 */
function baseSlice(array, start, end) {
  var index = -1,
      length = array.length;

  if (start < 0) {
    start = -start > length ? 0 : (length + start);
  }
  end = end > length ? length : end;
  if (end < 0) {
    end += length;
  }
  length = start > end ? 0 : ((end - start) >>> 0);
  start >>>= 0;

  var result = Array(length);
  while (++index < length) {
    result[index] = array[index + start];
  }
  return result;
}

var _baseSlice = baseSlice;

/**
 * Gets the parent value at `path` of `object`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array} path The path to get the parent value of.
 * @returns {*} Returns the parent value.
 */
function parent(object, path) {
  return path.length < 2 ? object : baseGet(object, _baseSlice(path, 0, -1));
}

var _parent = parent;

/**
 * The base implementation of `_.unset`.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {Array|string} path The property path to unset.
 * @returns {boolean} Returns `true` if the property is deleted, else `false`.
 */
function baseUnset(object, path) {
  path = castPath(path, object);
  object = _parent(object, path);
  return object == null || delete object[toKey(last_1(path))];
}

var _baseUnset = baseUnset;

/** `Object#toString` result references. */
var objectTag = '[object Object]';

/** Used for built-in method references. */
var funcProto = Function.prototype,
    objectProto$1 = Object.prototype;

/** Used to resolve the decompiled source of functions. */
var funcToString = funcProto.toString;

/** Used to check objects for own properties. */
var hasOwnProperty$1 = objectProto$1.hasOwnProperty;

/** Used to infer the `Object` constructor. */
var objectCtorString = funcToString.call(Object);

/**
 * Checks if `value` is a plain object, that is, an object created by the
 * `Object` constructor or one with a `[[Prototype]]` of `null`.
 *
 * @static
 * @memberOf _
 * @since 0.8.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a plain object, else `false`.
 * @example
 *
 * function Foo() {
 *   this.a = 1;
 * }
 *
 * _.isPlainObject(new Foo);
 * // => false
 *
 * _.isPlainObject([1, 2, 3]);
 * // => false
 *
 * _.isPlainObject({ 'x': 0, 'y': 0 });
 * // => true
 *
 * _.isPlainObject(Object.create(null));
 * // => true
 */
function isPlainObject(value) {
  if (!isObjectLike(value) || baseGetTag(value) != objectTag) {
    return false;
  }
  var proto = getPrototype(value);
  if (proto === null) {
    return true;
  }
  var Ctor = hasOwnProperty$1.call(proto, 'constructor') && proto.constructor;
  return typeof Ctor == 'function' && Ctor instanceof Ctor &&
    funcToString.call(Ctor) == objectCtorString;
}

var isPlainObject_1 = isPlainObject;

/**
 * Used by `_.omit` to customize its `_.cloneDeep` use to only clone plain
 * objects.
 *
 * @private
 * @param {*} value The value to inspect.
 * @param {string} key The key of the property to inspect.
 * @returns {*} Returns the uncloned value or `undefined` to defer cloning to `_.cloneDeep`.
 */
function customOmitClone(value) {
  return isPlainObject_1(value) ? undefined : value;
}

var _customOmitClone = customOmitClone;

/** Built-in value references. */
var spreadableSymbol = Symbol ? Symbol.isConcatSpreadable : undefined;

/**
 * Checks if `value` is a flattenable `arguments` object or array.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is flattenable, else `false`.
 */
function isFlattenable(value) {
  return isArray(value) || isArguments(value) ||
    !!(spreadableSymbol && value && value[spreadableSymbol]);
}

var _isFlattenable = isFlattenable;

/**
 * The base implementation of `_.flatten` with support for restricting flattening.
 *
 * @private
 * @param {Array} array The array to flatten.
 * @param {number} depth The maximum recursion depth.
 * @param {boolean} [predicate=isFlattenable] The function invoked per iteration.
 * @param {boolean} [isStrict] Restrict to values that pass `predicate` checks.
 * @param {Array} [result=[]] The initial result value.
 * @returns {Array} Returns the new flattened array.
 */
function baseFlatten(array, depth, predicate, isStrict, result) {
  var index = -1,
      length = array.length;

  predicate || (predicate = _isFlattenable);
  result || (result = []);

  while (++index < length) {
    var value = array[index];
    if (depth > 0 && predicate(value)) {
      if (depth > 1) {
        // Recursively flatten arrays (susceptible to call stack limits).
        baseFlatten(value, depth - 1, predicate, isStrict, result);
      } else {
        arrayPush(result, value);
      }
    } else if (!isStrict) {
      result[result.length] = value;
    }
  }
  return result;
}

var _baseFlatten = baseFlatten;

/**
 * Flattens `array` a single level deep.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to flatten.
 * @returns {Array} Returns the new flattened array.
 * @example
 *
 * _.flatten([1, [2, [3, [4]], 5]]);
 * // => [1, 2, [3, [4]], 5]
 */
function flatten(array) {
  var length = array == null ? 0 : array.length;
  return length ? _baseFlatten(array, 1) : [];
}

var flatten_1 = flatten;

/**
 * A faster alternative to `Function#apply`, this function invokes `func`
 * with the `this` binding of `thisArg` and the arguments of `args`.
 *
 * @private
 * @param {Function} func The function to invoke.
 * @param {*} thisArg The `this` binding of `func`.
 * @param {Array} args The arguments to invoke `func` with.
 * @returns {*} Returns the result of `func`.
 */
function apply(func, thisArg, args) {
  switch (args.length) {
    case 0: return func.call(thisArg);
    case 1: return func.call(thisArg, args[0]);
    case 2: return func.call(thisArg, args[0], args[1]);
    case 3: return func.call(thisArg, args[0], args[1], args[2]);
  }
  return func.apply(thisArg, args);
}

var _apply = apply;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max;

/**
 * A specialized version of `baseRest` which transforms the rest array.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @param {number} [start=func.length-1] The start position of the rest parameter.
 * @param {Function} transform The rest array transform.
 * @returns {Function} Returns the new function.
 */
function overRest(func, start, transform) {
  start = nativeMax(start === undefined ? (func.length - 1) : start, 0);
  return function() {
    var args = arguments,
        index = -1,
        length = nativeMax(args.length - start, 0),
        array = Array(length);

    while (++index < length) {
      array[index] = args[start + index];
    }
    index = -1;
    var otherArgs = Array(start + 1);
    while (++index < start) {
      otherArgs[index] = args[index];
    }
    otherArgs[start] = transform(array);
    return _apply(func, this, otherArgs);
  };
}

var _overRest = overRest;

/**
 * Creates a function that returns `value`.
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Util
 * @param {*} value The value to return from the new function.
 * @returns {Function} Returns the new constant function.
 * @example
 *
 * var objects = _.times(2, _.constant({ 'a': 1 }));
 *
 * console.log(objects);
 * // => [{ 'a': 1 }, { 'a': 1 }]
 *
 * console.log(objects[0] === objects[1]);
 * // => true
 */
function constant(value) {
  return function() {
    return value;
  };
}

var constant_1 = constant;

/**
 * The base implementation of `setToString` without support for hot loop shorting.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var baseSetToString = !defineProperty ? identity : function(func, string) {
  return defineProperty(func, 'toString', {
    'configurable': true,
    'enumerable': false,
    'value': constant_1(string),
    'writable': true
  });
};

var _baseSetToString = baseSetToString;

/** Used to detect hot functions by number of calls within a span of milliseconds. */
var HOT_COUNT = 800,
    HOT_SPAN = 16;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeNow = Date.now;

/**
 * Creates a function that'll short out and invoke `identity` instead
 * of `func` when it's called `HOT_COUNT` or more times in `HOT_SPAN`
 * milliseconds.
 *
 * @private
 * @param {Function} func The function to restrict.
 * @returns {Function} Returns the new shortable function.
 */
function shortOut(func) {
  var count = 0,
      lastCalled = 0;

  return function() {
    var stamp = nativeNow(),
        remaining = HOT_SPAN - (stamp - lastCalled);

    lastCalled = stamp;
    if (remaining > 0) {
      if (++count >= HOT_COUNT) {
        return arguments[0];
      }
    } else {
      count = 0;
    }
    return func.apply(undefined, arguments);
  };
}

var _shortOut = shortOut;

/**
 * Sets the `toString` method of `func` to return `string`.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var setToString = _shortOut(_baseSetToString);

var _setToString = setToString;

/**
 * A specialized version of `baseRest` which flattens the rest array.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @returns {Function} Returns the new function.
 */
function flatRest(func) {
  return _setToString(_overRest(func, undefined, flatten_1), func + '');
}

var _flatRest = flatRest;

/** Used to compose bitmasks for cloning. */
var CLONE_DEEP_FLAG = 1,
    CLONE_FLAT_FLAG = 2,
    CLONE_SYMBOLS_FLAG = 4;

/**
 * The opposite of `_.pick`; this method creates an object composed of the
 * own and inherited enumerable property paths of `object` that are not omitted.
 *
 * **Note:** This method is considerably slower than `_.pick`.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Object
 * @param {Object} object The source object.
 * @param {...(string|string[])} [paths] The property paths to omit.
 * @returns {Object} Returns the new object.
 * @example
 *
 * var object = { 'a': 1, 'b': '2', 'c': 3 };
 *
 * _.omit(object, ['a', 'c']);
 * // => { 'b': '2' }
 */
var omit = _flatRest(function(object, paths) {
  var result = {};
  if (object == null) {
    return result;
  }
  var isDeep = false;
  paths = arrayMap(paths, function(path) {
    path = castPath(path, object);
    isDeep || (isDeep = path.length > 1);
    return path;
  });
  copyObject(object, getAllKeysIn(object), result);
  if (isDeep) {
    result = baseClone(result, CLONE_DEEP_FLAG | CLONE_FLAT_FLAG | CLONE_SYMBOLS_FLAG, _customOmitClone);
  }
  var length = paths.length;
  while (length--) {
    _baseUnset(result, paths[length]);
  }
  return result;
});

var omit_1 = omit;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeCeil = Math.ceil,
    nativeMax$1 = Math.max;

/**
 * The base implementation of `_.range` and `_.rangeRight` which doesn't
 * coerce arguments.
 *
 * @private
 * @param {number} start The start of the range.
 * @param {number} end The end of the range.
 * @param {number} step The value to increment or decrement by.
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {Array} Returns the range of numbers.
 */
function baseRange(start, end, step, fromRight) {
  var index = -1,
      length = nativeMax$1(nativeCeil((end - start) / (step || 1)), 0),
      result = Array(length);

  while (length--) {
    result[fromRight ? length : ++index] = start;
    start += step;
  }
  return result;
}

var _baseRange = baseRange;

/**
 * Checks if the given arguments are from an iteratee call.
 *
 * @private
 * @param {*} value The potential iteratee value argument.
 * @param {*} index The potential iteratee index or key argument.
 * @param {*} object The potential iteratee object argument.
 * @returns {boolean} Returns `true` if the arguments are from an iteratee call,
 *  else `false`.
 */
function isIterateeCall(value, index, object) {
  if (!isObject(object)) {
    return false;
  }
  var type = typeof index;
  if (type == 'number'
        ? (isArrayLike(object) && isIndex(index, object.length))
        : (type == 'string' && index in object)
      ) {
    return eq(object[index], value);
  }
  return false;
}

var _isIterateeCall = isIterateeCall;

/** Used to match a single whitespace character. */
var reWhitespace = /\s/;

/**
 * Used by `_.trim` and `_.trimEnd` to get the index of the last non-whitespace
 * character of `string`.
 *
 * @private
 * @param {string} string The string to inspect.
 * @returns {number} Returns the index of the last non-whitespace character.
 */
function trimmedEndIndex(string) {
  var index = string.length;

  while (index-- && reWhitespace.test(string.charAt(index))) {}
  return index;
}

var _trimmedEndIndex = trimmedEndIndex;

/** Used to match leading whitespace. */
var reTrimStart = /^\s+/;

/**
 * The base implementation of `_.trim`.
 *
 * @private
 * @param {string} string The string to trim.
 * @returns {string} Returns the trimmed string.
 */
function baseTrim(string) {
  return string
    ? string.slice(0, _trimmedEndIndex(string) + 1).replace(reTrimStart, '')
    : string;
}

var _baseTrim = baseTrim;

/** Used as references for various `Number` constants. */
var NAN = 0 / 0;

/** Used to detect bad signed hexadecimal string values. */
var reIsBadHex = /^[-+]0x[0-9a-f]+$/i;

/** Used to detect binary string values. */
var reIsBinary = /^0b[01]+$/i;

/** Used to detect octal string values. */
var reIsOctal = /^0o[0-7]+$/i;

/** Built-in method references without a dependency on `root`. */
var freeParseInt = parseInt;

/**
 * Converts `value` to a number.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to process.
 * @returns {number} Returns the number.
 * @example
 *
 * _.toNumber(3.2);
 * // => 3.2
 *
 * _.toNumber(Number.MIN_VALUE);
 * // => 5e-324
 *
 * _.toNumber(Infinity);
 * // => Infinity
 *
 * _.toNumber('3.2');
 * // => 3.2
 */
function toNumber(value) {
  if (typeof value == 'number') {
    return value;
  }
  if (isSymbol(value)) {
    return NAN;
  }
  if (isObject(value)) {
    var other = typeof value.valueOf == 'function' ? value.valueOf() : value;
    value = isObject(other) ? (other + '') : other;
  }
  if (typeof value != 'string') {
    return value === 0 ? value : +value;
  }
  value = _baseTrim(value);
  var isBinary = reIsBinary.test(value);
  return (isBinary || reIsOctal.test(value))
    ? freeParseInt(value.slice(2), isBinary ? 2 : 8)
    : (reIsBadHex.test(value) ? NAN : +value);
}

var toNumber_1 = toNumber;

/** Used as references for various `Number` constants. */
var INFINITY = 1 / 0,
    MAX_INTEGER = 1.7976931348623157e+308;

/**
 * Converts `value` to a finite number.
 *
 * @static
 * @memberOf _
 * @since 4.12.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {number} Returns the converted number.
 * @example
 *
 * _.toFinite(3.2);
 * // => 3.2
 *
 * _.toFinite(Number.MIN_VALUE);
 * // => 5e-324
 *
 * _.toFinite(Infinity);
 * // => 1.7976931348623157e+308
 *
 * _.toFinite('3.2');
 * // => 3.2
 */
function toFinite(value) {
  if (!value) {
    return value === 0 ? value : 0;
  }
  value = toNumber_1(value);
  if (value === INFINITY || value === -INFINITY) {
    var sign = (value < 0 ? -1 : 1);
    return sign * MAX_INTEGER;
  }
  return value === value ? value : 0;
}

var toFinite_1 = toFinite;

/**
 * Creates a `_.range` or `_.rangeRight` function.
 *
 * @private
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {Function} Returns the new range function.
 */
function createRange(fromRight) {
  return function(start, end, step) {
    if (step && typeof step != 'number' && _isIterateeCall(start, end, step)) {
      end = step = undefined;
    }
    // Ensure the sign of `-0` is preserved.
    start = toFinite_1(start);
    if (end === undefined) {
      end = start;
      start = 0;
    } else {
      end = toFinite_1(end);
    }
    step = step === undefined ? (start < end ? 1 : -1) : toFinite_1(step);
    return _baseRange(start, end, step, fromRight);
  };
}

var _createRange = createRange;

/**
 * Creates an array of numbers (positive and/or negative) progressing from
 * `start` up to, but not including, `end`. A step of `-1` is used if a negative
 * `start` is specified without an `end` or `step`. If `end` is not specified,
 * it's set to `start` with `start` then set to `0`.
 *
 * **Note:** JavaScript follows the IEEE-754 standard for resolving
 * floating-point values which can produce unexpected results.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {number} [start=0] The start of the range.
 * @param {number} end The end of the range.
 * @param {number} [step=1] The value to increment or decrement by.
 * @returns {Array} Returns the range of numbers.
 * @see _.inRange, _.rangeRight
 * @example
 *
 * _.range(4);
 * // => [0, 1, 2, 3]
 *
 * _.range(-4);
 * // => [0, -1, -2, -3]
 *
 * _.range(1, 5);
 * // => [1, 2, 3, 4]
 *
 * _.range(0, 20, 5);
 * // => [0, 5, 10, 15]
 *
 * _.range(0, -4, -1);
 * // => [0, -1, -2, -3]
 *
 * _.range(1, 4, 0);
 * // => [1, 1, 1]
 *
 * _.range(0);
 * // => []
 */
var range = _createRange();

var range_1 = range;

class DataURLImageSupport {
    insert(file, done, progressFn) {
        const reader = new FileReader();
        reader.onload = () => {
            const dataURL = reader.result;
            setTimeout(() => {
                done(null, dataURL.toString());
            }, 2000);
        };
        let progress = 0;
        progressFn(progress, 0, 100);
        range_1(1, 100).forEach(n => {
            setTimeout(() => {
                progressFn(n, n, 100);
            }, n * 20);
        });
        // if external asset support .. add reader.readAsArrayBuffer ..
        reader.readAsDataURL(file);
    }
    delete(src, done) {
        done();
    }
}

class DataURLUploadSoundSupport {
    insert(file, done, progressFn) {
        const reader = new FileReader();
        reader.onload = () => {
            const dataURL = reader.result;
            setTimeout(() => {
                done(null, dataURL.toString());
            }, 2000);
        };
        let progress = 0;
        progressFn(progress, 0, 100);
        range_1(1, 100).forEach(n => {
            setTimeout(() => {
                progressFn(n, n, 100);
            }, n * 20);
        });
        // if external asset support .. add reader.readAsArrayBuffer ..
        reader.readAsDataURL(file);
    }
    delete(src, done) {
        done();
    }
}

const Author = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this._modelLoadedState = false;
        /**
         * Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements
         * used by the item config.
         * This if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers.
         */
        this.disableBundler = false;
        /**
         * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
         */
        this.addPreview = false;
        this.elementsLoaded = false;
        /**
         * If pie-author is used inside pie-api-author component. Do not set it manually.
         */
        this.isInsidePieApiAuthor = false;
        this.pieLoader = new PieLoader(null, this.loaderConfig);
        this.fileInput = null;
        this.imageHandler = null;
        this.dialogOpened = false;
        this.changeHandled = false;
        this.componentLoaded = false;
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.imageSupport = new DataURLImageSupport();
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.uploadSoundSupport = new DataURLUploadSoundSupport();
        this.version = VERSION;
        /**
         * used in our demo environment to allow author to watch config settings and make updates
         * defaults to false (do not set it to true because it was not tested properly)
         * @type {boolean}
         * @private
         */
        this.canWatchConfigSettings = false;
        /**
         * used to automatically re-fetch the bundle (in case we get a 503)
         */
        this.reFetchBundle = false;
        this.isAdvancedItemConfig = (config) => config.pie;
        this.getElementByType = (elements, type) => {
            const result = [];
            for (const [key, value] of Object.entries(elements || {})) {
                if (value.startsWith(type)) {
                    result.push(key);
                }
            }
            return result;
        };
        this.getPieContentModelWithToggledComplexRubric = async ({ complexRubricCheckedValues, pieContentModel }) => {
            const { shouldAddComplexRubric, shouldRemoveComplexRubric, complexRubricElements } = complexRubricCheckedValues || {};
            if (shouldAddComplexRubric) {
                let clonedModel = cloneDeep(pieContentModel);
                // we have to convert the rubric to complex-rubric, so get the default values from it
                const rubricNames = this.getElementByType(pieContentModel.elements, "@pie-element/rubric");
                for (const rubricName of rubricNames) {
                    const rubric = ((pieContentModel && pieContentModel.models) || []).find(el => el.element === rubricName);
                    if (rubric) {
                        const simpleRubric = omit_1(rubric, ["id", "element"]);
                        this.defaultComplexRubricModel = {
                            rubrics: {
                                simpleRubric
                            }
                        };
                        clonedModel = this.removeRubricItemTypes(clonedModel, rubric.id, rubric.element);
                    }
                }
                // we add complex-rubric to config
                return this.addComplexRubric(clonedModel);
            }
            if (shouldRemoveComplexRubric) {
                // we remove complex-rubric from config
                return this.removeComplexRubricItemTypes(cloneDeep(pieContentModel), complexRubricElements);
            }
            return null;
        };
        this.getNewConfig = newConfig => {
            if (this.isAdvancedItemConfig(this.config)) {
                return Object.assign(Object.assign({}, this.config), { pie: newConfig });
            }
            return newConfig;
        };
        this.setModelsAndConfigurations = () => {
            if (this.pieContentModel && this.pieContentModel.models) {
                this.pieContentModel.models.map(model => {
                    let pieEl = this.el.querySelector(`[id='${model.id}']`);
                    !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
                    if (pieEl) {
                        const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                        try {
                            const packageName = parseNpm(this.pieContentModel.elements[pieElName]).name;
                            pieEl.model = model;
                            if (this.configSettings && this.configSettings[packageName]) {
                                pieEl.configuration = this.configSettings[packageName];
                            }
                        }
                        catch (e) {
                            console.log(e.toString(), pieElName, this.pieContentModel.elements[pieElName]);
                        }
                    }
                });
            }
        };
        this.handleFileInputChange = (e) => {
            const input = e.target;
            this.dialogOpened = false;
            if (!this.imageHandler) {
                console.error("no image handler - but file input change triggered?");
                return;
            }
            this.changeHandled = true;
            const files = input.files;
            if (files.length < 1 || !files[0]) {
                this.imageHandler.cancel();
                this.imageHandler = null;
            }
            else {
                const file = files[0];
                this.imageHandler.fileChosen(file);
                this.fileInput.value = "";
                this.insertImage(file);
            }
        };
        this.handleWindowFocus = () => {
            if (!this.fileInput || !this.dialogOpened) {
                return;
            }
            this.dialogOpened = false;
            // needed in order for the fileInput to be populated with the file
            setTimeout(() => {
                if (this.fileInput.files.length === 0 && !this.changeHandled) {
                    // it means the user clicked cancel
                    this.imageHandler.cancel();
                    this.imageHandler = null;
                    this.changeHandled = true;
                }
            }, 500);
        };
        this.handleInsertImage = (e) => {
            console.log("[handleInsertImage]", e);
            this.imageHandler = e.detail;
            if (!e.detail.isPasted) {
                this.fileInput.click();
                this.dialogOpened = true;
                this.changeHandled = false;
            }
            else if (e.detail.getChosenFile) {
                // this is for images that were pasted into the editor (or dropped)
                // they also need to be uploaded, but the file input doesn't have to be used
                try {
                    const file = e.detail.getChosenFile();
                    if (file) {
                        this.insertImage(file);
                    }
                }
                catch (e) {
                    console.error(e);
                }
            }
        };
        this.handleDeleteImage = (e) => {
            console.log("[handleDeleteImage ..]", e);
            this.imageSupport.delete(e.detail.src, e.detail.done);
        };
        this.handleInsertSound = (e) => {
            console.log("[handleInsertSound]", e);
            this.uploadSoundSupport.insert(e.detail.fileChosen, e.detail.done);
        };
        this.handleDeleteSound = (e) => {
            console.log("[handleDeleteSound ..]", e);
            this.uploadSoundSupport.delete(e.detail.src, e.detail.done);
        };
        this.modelUpdated = createEvent(this, "modelUpdated", 7);
        this.modelLoaded = createEvent(this, "modelLoaded", 7);
        this.doc = getContext(this, "document");
    }
    async validateModels() {
        if (!this.pieContentModel || !this.pieContentModel.models) {
            console.error("No pie content model");
            return { hasErrors: false, validatedModels: {} };
        }
        return (this.pieContentModel.models || []).reduce((acc, model, index) => {
            let pieEl = this.el.querySelector(`[id='${model.id}']`);
            !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
            if (pieEl) {
                const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                const controller = this.pieLoader.getController(pieElName);
                const packageName = parseNpm(this.pieContentModel.elements[pieElName])
                    .name;
                const configuration = (this.configSettings && this.configSettings[packageName]) || {};
                if (controller && controller.validate) {
                    // here we call controller.validate which returns an object with all the errors
                    const errors = controller.validate(model, configuration);
                    const errorsAreTheSame = isEqual_1(model && model.errors, errors);
                    // Only update models if anything changed to errors object, otherwise, we'll keep calling onModelUpdated and again validateModels
                    // for complex items (like ebsr, complex-rubric, who are using other items)
                    if (!errorsAreTheSame) {
                        // added this to ensure that this.pieContentModel.model is updated with the correct errors
                        // TODO: figure out what's actually happening and if there is a better way to make sure that this.pieContentModels.models is actually updated
                        this.pieContentModel.models[index] = Object.assign(Object.assign({}, model), { errors });
                        // here we can update the model in author, so we can set errors
                        pieEl.model = Object.assign(Object.assign({}, model), { errors });
                    }
                    // for ebsr
                    if (errors && errors.partA && errors.partB) {
                        acc.hasErrors =
                            acc.hasErrors ||
                                (!isEmpty_1(errors.partA) || !isEmpty_1(errors.partB));
                    }
                    else {
                        // here we return a boolean value if models are valid or not
                        acc.hasErrors = acc.hasErrors || !isEmpty_1(errors);
                    }
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: Object.assign(Object.assign({}, model), { errors }) });
                }
                else {
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: model });
                }
            }
            return acc;
        }, { hasErrors: false, validatedModels: {} });
    }
    insertImage(file) {
        this.imageSupport.insert(file, (e, src) => {
            if (e) {
                console.warn("error inserting image: ", e.message);
                console.error(e);
            }
            this.imageHandler.done(e, src);
            this.imageHandler = null;
        }, (percent, bytes, total) => this.imageHandler.progress(percent, bytes, total));
    }
    getRenderMarkup() {
        let markup = this.pieContentModel ? this.pieContentModel.markup : "";
        if (markup) {
            Object.keys(this.pieContentModel.elements).forEach(key => {
                markup = markup.split(key).join(key + "-config");
            });
            return markup;
        }
    }
    async watchAddPreview() {
        // if add preview changes, make sure to reset the configuration
        // this is not a common use case
        if (this.componentLoaded) {
            await this.watchConfig(this.config, {});
        }
    }
    async watchConfig(newValue, oldValue) {
        if (newValue && !isEqual_1(newValue, oldValue)) {
            try {
                this.elementsLoaded = false;
                this._modelLoadedState = false;
                const pieContentModel = pieContentFromConfig(newValue);
                const complexRubricCheckedValues = complexRubricChecks(pieContentModel, this.configSettings);
                const { shouldAddComplexRubric, shouldRemoveComplexRubric } = complexRubricCheckedValues || {};
                // if changes are needed
                if (shouldAddComplexRubric || shouldRemoveComplexRubric) {
                    this.pieContentModel = null;
                    const newConfig = await this.getPieContentModelWithToggledComplexRubric({
                        complexRubricCheckedValues,
                        pieContentModel
                    });
                    // and then we reset the config
                    if (newConfig) {
                        this.config = this.getNewConfig(newConfig);
                    }
                }
                else {
                    // if there's no change needed, we can proceed with updating the config and loading pie-elements
                    this.pieContentModel = pieContentModel;
                    this.addConfigTags(this.pieContentModel);
                    await this.loadPieElements();
                }
            }
            catch (error) {
                console.log(`ERROR ${error}`);
            }
        }
    }
    removeRubricItemTypes(pieContentModel, rubricId, rubricElement) {
        if (!rubricElement || !pieContentModel.models) {
            return pieContentModel;
        }
        // delete the rubric elements from elements object
        delete pieContentModel.elements[rubricElement];
        const { markupWithoutComplexRubric } = removeComplexRubricFromMarkup(pieContentModel, [rubricElement], this.doc);
        // delete the rubric nodes from markup
        pieContentModel.markup = markupWithoutComplexRubric;
        // delete the rubric models
        pieContentModel.models = (pieContentModel.models || []).filter(model => rubricId !== model.id);
        return pieContentModel;
    }
    async removeComplexRubricItemTypes(pieContentModel, rubricElements) {
        if (!rubricElements.length || !pieContentModel.models) {
            return pieContentModel;
        }
        // delete the complex-rubric elements from elements object
        rubricElements.forEach(rubricElementKey => delete pieContentModel.elements[rubricElementKey]);
        const { markupWithoutComplexRubric, deletedComplexRubricItemIds } = removeComplexRubricFromMarkup(pieContentModel, rubricElements, this.doc);
        // delete the complex-rubric nodes from markup
        pieContentModel.markup = markupWithoutComplexRubric;
        // delete the complex-rubric models
        pieContentModel.models = pieContentModel.models.filter(model => !deletedComplexRubricItemIds.includes(model.id));
        return pieContentModel;
    }
    async addComplexRubric(pieContentModel) {
        const complexRubricModel = Object.assign({ id: COMPLEX_RUBRIC, element: `pie-${COMPLEX_RUBRIC}` }, (this.defaultComplexRubricModel || {}));
        const packageName = `@pie-element/${COMPLEX_RUBRIC}`;
        // add complex-rubric in elements, IF it isn't there already
        let content = addPackageToContent(pieContentModel, packageName, complexRubricModel);
        // but if complex-rubric is already in elements, then just add it in the models
        if (!content) {
            content = addModelToContent(pieContentModel, createTag(packageName), complexRubricModel);
        }
        return addComplexRubric(content);
    }
    addConfigTags(c) {
        if (!c.markup && c.models) {
            const tags = c.models.map(model => {
                return `<${model.element}-config id="${model.id}"></${model.element}-config>`;
            });
            c.markup = tags.join("");
        }
    }
    async watchElementsLoaded(newValue, oldValue) {
        if (newValue && !oldValue) {
            await this.updateModels();
        }
    }
    async updateModels() {
        if (this.pieContentModel &&
            this.pieContentModel.elements &&
            this.pieContentModel.markup) {
            if (!this.pieContentModel.models) {
                this.pieContentModel.models = [];
            }
            const tempDiv = this.doc.createElement("div");
            tempDiv.innerHTML = this.pieContentModel.markup;
            const elsWithId = tempDiv.querySelectorAll("[id]");
            // set up a model for each pie defined in the markup
            elsWithId.forEach(el => {
                const pieElName = el.tagName.toLowerCase().split("-config")[0];
                // initialize emtpy model if this is a pie
                if (this.pieContentModel.elements[pieElName]) {
                    const elementId = el.getAttribute("id");
                    if (!this.pieContentModel.models.find(m => m.id === elementId)) {
                        const model = { id: elementId, element: pieElName };
                        this.pieContentModel.models.push(model);
                    }
                }
            });
            tempDiv.remove();
        }
        if (this.pieContentModel && this.pieContentModel.models) {
            this.setModelsAndConfigurations();
            if (this._modelLoadedState === false) {
                this._modelLoadedState = true;
                this.modelLoaded.emit(this.pieContentModel);
            }
        }
    }
    watchConfigSettings() {
        if (this.canWatchConfigSettings) {
            this.setModelsAndConfigurations();
        }
    }
    componentDidUnload() {
        this.el.removeEventListener(lib_3.TYPE, this.handleInsertImage);
        this.el.removeEventListener(lib_2.TYPE, this.handleDeleteImage);
        this.el.removeEventListener(lib_5.TYPE, this.handleInsertSound);
        this.el.removeEventListener(lib_4.TYPE, this.handleDeleteSound);
        if (this.fileInput) {
            this.fileInput.removeEventListener("change", this.handleFileInputChange);
        }
        window.removeEventListener("focus", this.handleWindowFocus);
    }
    async componentWillLoad() {
        if (this.config) {
            this.watchConfig(this.config, {});
        }
        // Note: cannot use the @Listen decorator as creates bundling problems due
        // to `.` in event name.
        this.el.addEventListener(lib_1.TYPE, async (e) => {
            // set the internal model
            // emit a content-item level event with the model
            let rubricChanged;
            if (this.pieContentModel && e.update) {
                this.pieContentModel.models.forEach(m => {
                    if (m.id === e.update.id && m.element === e.update.element) {
                        rubricChanged =
                            rubricChanged || m.rubricEnabled !== e.update.rubricEnabled;
                        Object.assign(m, e.update);
                        if (e.update.errors && !isEmpty_1(e.update.errors)) {
                            this.validateModels();
                        }
                    }
                });
            }
            if (this._modelLoadedState) {
                if (rubricChanged) {
                    if (this.isInsidePieApiAuthor) {
                        // if pie-author is used inside pie-api-author, then what we need is to
                        // calculate the new pieContentModel and make sure we emit the modelUpdated event with the updated pieContentModel
                        // first thing, we check what changed (if we have to add or remove complex-rubric)
                        const complexRubricCheckedValues = complexRubricChecks(this.pieContentModel, this.configSettings);
                        const { shouldAddComplexRubric, shouldRemoveComplexRubric } = complexRubricCheckedValues || {};
                        if (shouldAddComplexRubric || shouldRemoveComplexRubric) {
                            // then, if changes are needed, we make them and save the new contentModel
                            const newPieContentModel = await this.getPieContentModelWithToggledComplexRubric({
                                complexRubricCheckedValues,
                                pieContentModel: this.pieContentModel
                            });
                            // last thing to do is to make sure we send the updated pie content model to pie-api-author
                            // (which listens for this event) who will trigger watchConfig
                            if (newPieContentModel) {
                                this.modelUpdated.emit(newPieContentModel);
                            }
                        }
                    }
                    else {
                        // however, if we only use pie-player-components, just force the call to watchConfig
                        await this.watchConfig(this.config, {});
                    }
                }
                else {
                    // if rubricEnabled did not change, just behave as previously
                    // (pie-api-author listens for this event and triggers watchConfig)
                    this.modelUpdated.emit(this.pieContentModel);
                }
            }
        });
        this.el.addEventListener(lib_3.TYPE, this.handleInsertImage);
        this.el.addEventListener(lib_2.TYPE, this.handleDeleteImage);
        this.el.addEventListener(lib_5.TYPE, this.handleInsertSound);
        this.el.addEventListener(lib_4.TYPE, this.handleDeleteSound);
        window.addEventListener("focus", this.handleWindowFocus);
    }
    async componentDidLoad() {
        this.componentLoaded = true;
        await this.afterRender();
    }
    async componentDidUpdate() {
        await this.afterRender();
        if (this.fileInput) {
            this.fileInput.addEventListener("change", this.handleFileInputChange);
        }
    }
    async loadPieElements() {
        if (this.config && !this.disableBundler) {
            let endpoints = DEFAULT_ENDPOINTS.prod;
            if (this.bundleHost &&
                ["dev", "stage", "prod"].includes(this.bundleHost)) {
                endpoints = DEFAULT_ENDPOINTS[this.bundleHost];
            }
            // if bundleEndpoints are provided, use them instead of the default
            // used for proxies, etc.
            let forceBundleUrl = false;
            if (this.bundleEndpoints) {
                endpoints = this.bundleEndpoints;
                forceBundleUrl = true;
            }
            await this.pieLoader.loadCloudPies({
                content: this.pieContentModel,
                doc: this.doc,
                endpoints,
                useCdn: false,
                forceBundleUrl,
                reFetchBundle: this.reFetchBundle
            });
        }
    }
    renderMath() {
        mathRenderingAccessible$1.renderMath(this.el);
    }
    async afterRender() {
        if (this.pieContentModel &&
            this.pieContentModel.markup &&
            !this.elementsLoaded) {
            const elements = Object.keys(this.pieContentModel.elements).map(el => ({
                name: el,
                tag: `${el}-config`
            }));
            const loadedInfo = await this.pieLoader.elementsHaveLoaded(elements);
            if (loadedInfo.val &&
                !!loadedInfo.elements.find(el => this.pieContentModel.elements[el.name])) {
                this.elementsLoaded = true;
                this.renderMath();
            }
        }
    }
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel
     */
    async addRubricToConfig(config, rubricModel) {
        console.warn("If you are using complex-rubric, stop using this function to prevent having duplicated rubrics.");
        if (!rubricModel) {
            rubricModel = {
                id: "rubric",
                element: "pie-rubric",
                points: ["", "", "", ""],
                maxPoints: 4,
                excludeZero: false
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/rubric", rubricModel);
        return addRubric(configPieContent);
    }
    /**
     * Utility method to add a `@pie-element/multi-trait-rubric` section to an item config when creating an item should be used before setting the config.
     **
     * @param config the item config to mutate
     * @param multiTraitRubricModel
     */
    async addMultiTraitRubricToConfig(config, multiTraitRubricModel) {
        console.warn("If you are using complex-rubric, stop using this function to prevent having duplicated rubrics.");
        if (!multiTraitRubricModel) {
            multiTraitRubricModel = {
                id: "multi-trait-rubric",
                element: "pie-multi-trait-rubric",
                visibleToStudent: true,
                halfScoring: false,
                excludeZero: true,
                pointLabels: true,
                description: false,
                standards: false,
                scales: [
                    {
                        maxPoints: 4,
                        scorePointsLabels: ["", "", "", ""],
                        traitLabel: "Trait",
                        traits: [
                            {
                                name: "",
                                standards: [],
                                description: "",
                                scorePointsDescriptors: ["", "", "", "", ""]
                            }
                        ]
                    }
                ]
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/multi-trait-rubric", multiTraitRubricModel);
        return addMultiTraitRubric(configPieContent);
    }
    render() {
        if (this.pieContentModel && this.pieContentModel.markup) {
            const markup = this.getRenderMarkup();
            if (this.addPreview) {
                return (h("pie-preview-layout", { config: this.config }, h("div", { slot: "configure" }, h("pie-spinner", { active: !this.elementsLoaded }, h("div", { class: "author-container", innerHTML: markup }))), h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: "none" } })));
            }
            else {
                return (h("pie-spinner", { active: !this.elementsLoaded }, h("div", { class: "author-container", innerHTML: markup }), h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: "none" } })));
            }
        }
        else {
            return h("pie-spinner", null);
        }
    }
    get el() { return getElement(this); }
    static get watchers() { return {
        "addPreview": ["watchAddPreview"],
        "config": ["watchConfig"],
        "elementsLoaded": ["watchElementsLoaded"],
        "configSettings": ["watchConfigSettings"]
    }; }
    static get style() { return ".pie-loading{position:absolute;height:100%;width:100%;background-color:#fff;bottom:0;left:0;right:0;top:0;z-index:9999;opacity:.4}.pie-loading.fixed{position:fixed}.pie-loading:before{-webkit-box-shadow:0 0 35px rgba(63,81,181,.9);box-shadow:0 0 35px rgba(63,81,181,.9);width:50px;height:50px;-moz-animation:spinPulse 1s infinite ease-in-out;-webkit-animation:spinPulse 1s linear infinite;margin:-25px 0 0 -25px}.pie-loading:after,.pie-loading:before{content:\"\";background-color:transparent;border:5px solid rgba(63,81,181,.9);opacity:.9;border-right:5px solid transparent;border-left:5px solid transparent;border-radius:50px;position:absolute;top:50%;left:50%}.pie-loading:after{-webkit-box-shadow:0 0 15px rgba(63,81,181,.9);box-shadow:0 0 15px rgba(63,81,181,.9);width:30px;height:30px;-moz-animation:spinoffPulse 1s infinite linear;-webkit-animation:spinoffPulse 1s linear infinite;margin:-15px 0 0 -15px}\@-webkit-keyframes spinPulse{0%{-webkit-transform:rotate(160deg);transform:rotate(160deg);opacity:0;-webkit-box-shadow:0 0 1px rgba(63,81,181,.9);box-shadow:0 0 1px rgba(63,81,181,.9)}50%{-webkit-transform:rotate(145deg);transform:rotate(145deg);opacity:1}to{-webkit-transform:rotate(-320deg);transform:rotate(-320deg);opacity:0}}\@keyframes spinPulse{0%{-webkit-transform:rotate(160deg);transform:rotate(160deg);opacity:0;-webkit-box-shadow:0 0 1px rgba(63,81,181,.9);box-shadow:0 0 1px rgba(63,81,181,.9)}50%{-webkit-transform:rotate(145deg);transform:rotate(145deg);opacity:1}to{-webkit-transform:rotate(-320deg);transform:rotate(-320deg);opacity:0}}\@-webkit-keyframes spinoffPulse{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}\@keyframes spinoffPulse{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}.pie-api-player img{max-width:inherit!important;max-height:inherit!important}.book-title-k5{text-decoration:underline}.book-title{font-style:italic}.block-quote,.text-block{font-family:serif;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;border:1px solid #000;padding:2px;display:block}.short-quote{font-weight:700}span.short-quote:before{content:open-quote}span.short-quote:after{content:close-quote}.word-callout{text-decoration:underline}.relative-emphasis{text-transform:uppercase}.content-emphasis{font-weight:700;color:red}span.passage-title:before{content:open-quote}span.passage-title:after{content:close-quote}.proper-name,.variable{font-style:italic}.equation-block{text-align:center;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;padding:2px;display:block}.embedded-error{font-style:italic}div.passage-subtitle,div.passage-title{font-weight:700;text-align:center}div.passage-author{text-align:center}.p-number{float:left;font-size:.85em}.numbered-paragraph{margin-left:36px}\@media print{.kds-noprint,.noprint{display:none}}.abs,.kds-absolute{border-left:1px solid #000;border-right:1px solid #000;padding:0 .3em 0 .3em}.kds-border-1{border:1px solid #000}.kds-border-2{border:2px solid #000}.center,.kds-center{margin:0;text-align:center}.kds-exponent{font-size:85%}.kds-exponent,sup.frac{margin-left:-.4em;position:relative;top:-1.4em}sup.frac{font-size:70%}.kds-fillin{display:inline-block}.fillin,.kds-fillin{border:1px solid #000;margin:0 .2em;padding-left:1em}.fillin{display:inline-block;*display:inline}.kds-font-larger{font-size:larger}.kds-font-smaller{font-size:smaller}table.kds-fraction{border:0;display:inline-block;font-size:85%;margin:0;padding:0;vertical-align:middle}table.kds-fraction>tbody>tr>td.kds-numerator{border-bottom:1px solid #000;border-left:0;border-right:0;border-top:0;text-align:center}table.kds-fraction>tbody>tr>td.kds-denominator{border:0;text-align:center}table.frac{display:inline-block;*display:inline;vertical-align:middle;font-size:85%;border:0;margin:0;padding:0}.frac .nu{border-left:0;border-top:0;border-right:0;border-bottom:1px solid #000;text-align:center}.frac .de{border:0;text-align:center}.kds-hanging-indent{margin:0;padding-left:3em;text-indent:-3em}.kds-image-caption{font-family:Verdana,Arial,sans-serif;text-align:center}.kds-image-left{float:left;margin-right:5px}.kds-image-right{float:right;margin-left:5px}.indent,.kds-indent{text-indent:3em}.kds-nowrap{display:inline-block;white-space:nowrap}.kds-overline{text-decoration:overline}.kds-parentheses{font-family:Arial Narrow,Arial,sans-serif}.kds-parentheses,b.frac{font-size:300%;font-weight:400;position:relative;top:8px}b.frac{font-family:arial narrow}.kds-pi{font-family:Times New Roman,serif;font-size:1.2em}.newradical{border-collapse:collapse;display:inline-block;*display:inline;vertical-align:middle}.newradical td{border:0;padding:0;vertical-align:top}.newradical td.vinculum{border-top:1px solid #000;padding:0;vertical-align:top}.newradical div.exp{font-size:.7em;position:relative;left:7px;bottom:5px}.kds-underline,.under{text-decoration:underline}.kds-verdana2t,.Verdana2t{border:1px solid #fff;text-align:left;text-decoration:none;vertical-align:top}.kds-whole-number{display:inline-block}.kds-whole-number,.whole{font-size:120%;margin:0 0 0 .2em}.whole{display:inline-block;*display:inline}table.KdsTable01{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable01>tbody>tr>th{font-weight:400}table.KdsTable01>tbody>tr>th,table.KdsTable01>tbody>tr>th.bold{background-color:#d3d3d3;border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>th.bold{font-weight:700}table.KdsTable01>tbody>tr>td,table.KdsTable01>tbody>tr>td.bold{border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>td.bold{font-weight:700}table.KdsTable02{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable02>tbody>tr>th{background-color:#d3d3d3;font-weight:400}table.KdsTable02>tbody>tr>td,table.KdsTable02>tbody>tr>th{border:1px solid #000;padding:2px}.table{width:100%;margin-bottom:1em;background-color:transparent;border-collapse:collapse}.table td,.table th{padding:.75em;vertical-align:top;border-top:1px solid #dee2e6}.table tbody+tbody{border-top:2px solid #dee2e6}.table thead th{vertical-align:bottom;border-bottom:2px solid #dee2e6}th{text-align:inherit}.table-bordered,.table-bordered td,.table-bordered th{border:1px solid #dee2e6}.table-bordered thead td,.table-bordered thead th{border-bottom-width:2px}.table-striped tbody tr:nth-of-type(odd){background-color:rgba(0,0,0,.05)}.text-center{text-align:center!important}.h5,h5{font-size:1.25em}h1,h2,h3,h4,h5,h6{margin-top:0}.h1,.h2,.h3,.h4,.h5,.h6,h1,h2,h3,h4,h5,h6{margin-bottom:.5em;font-family:inherit;font-weight:500;line-height:1.2;color:inherit}.lrn_feature h3{margin-top:0!important}#item,#stimulus{width:50%;float:left}.lrn_width_auto.table{width:auto}.TEX-S3{font-family:MJXZERO,MJXTEX-S3!important}.TEX-S2{font-family:MJXZERO,MJXTEX-S2!important}.TEX-S1{font-family:MJXZERO,MJXTEX-S1!important}.TEX-I{font-family:MJXZERO,MJXTEX-I!important}mjx-c.mjx-c2032{font-family:CerebriSans,serif!important}mjx-c.mjx-c22,mjx-c.mjx-c27{font-family:MJXZERO,serif!important}table{border-collapse:collapse}.evaluate-bottom-border>:first-child:after{content:\"\";display:block;border-bottom:1px solid #d3d3d3}"; }
};

export { Author as pie_author };
